<?php
/**
 * GetSimple plugin - Simply add contact forms in your pages
 *
 * This plugin let you add contact forms in your pages by writing simple tags.
 * You can also define recipients or create your own complex forms.
 *
 * Plugin page : http://nliautaud.fr/wiki/travaux/getsimple_p01-contact
 * Plugin repository : http://get-simple.info/extend/plugin/p01-contact/35
 * @author Nicolas Liautaud <contact@nliautaud.fr>
 * @package p01-contact
 * @version 0.8
 */

$thisfile = basename(__FILE__, '.php');

register_plugin(
	$thisfile, 	                // ID of plugin, should be filename minus php
	'p01-contact', 	            // Title of plugin
	'0.8', 		                // Version of plugin
	'Nicolas Liautaud',	        // Author of plugin
	'http://nliautaud.fr', 	    // Author URL
	'Simply add contact forms in your pages',// Plugin Description
	'plugins', 	                // Page type of plugin
	'p01contact_action_handle'  // Function that displays content
);

add_filter('content','p01contact_filter_handle');
add_action('plugins-sidebar','createSideMenu',array($thisfile,'p01-contact'));

$p01contact = new P01contact();

/*
 * Handle for GS filter (parse GS pages content)
 */
function p01contact_filter_handle($contents)
{
    global $p01contact;
    
    $contents = $p01contact->parse($contents);
    
    if($p01contact->settings('debug')) {
        $p01contact->debug();
    }
    return $contents;
}
/*
 * Handle for GS action (display admin panel)
 */
function p01contact_action_handle()
{
    global $p01contact;
    echo $p01contact->panel();
}

/*
 * Plugin main class
 
 * Parse page contents, create and manage contact
 * forms, manage plugin settings and languages.
 */
class P01contact 
{
    private $forms;

    public function __construct() 
    { 
        $this->forms = array();
    }
    
    /**
     * Parse a string to replace tags by forms
     *
     * Find tags, create forms structures, check POST and modify string.
     * @param string $contents the string to parse
     * @return string the modified string
     */
    public function parse($contents)
    {
        $pattern = '`(?<!<code>)\(%\s*contact\s*(.*)\s*%\)`';  
        preg_match_all($pattern, $contents, $tags, PREG_SET_ORDER);
        $ids = array();
        
        // create forms structures from TAG
        foreach($tags as $tag) {
            $id = $this->new_form_id();
            $form = $this->parse_tag($id, $tag[1]);
            $this->forms[$id] = $form;
            array_push($ids, $id); // forms manipulated by this parsing session
        }
        // modify forms structures from POST, send mail
        if(!empty($_POST['p01contact_form'])) {
            $this->post();
        }
        // replace tags by forms
        foreach($ids as $id) {
            $contents = preg_replace($pattern, $this->forms[$id]->html(), $contents, 1);            
        }
        return $contents;
    }
    
    /**
     * Parse a tag to create form structure
     *
     * Find emails and parameters, create and setup form object.
     * @param int $id the form id
     * @param string $tag the tag to parse
     * @return P01contact_form the form object
     */
    private function parse_tag($id, $tag)
    {
        $form = new P01contact_form($this, $id);
        
        $tag = trim(preg_replace('`&nbsp;`',' ', $tag));
        $param_pattern = '`[,:]\s*([^ ,!]+)(!)?\s*(&quot;([^&]*)&quot;)?\s*((=&gt;|=)?\s*([^,]*))?\s*`';
        $targets_pattern = '`[,:]\s*([_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3}))`i';
        
        // parse emails
        preg_match_all($targets_pattern, $tag, $targets);
        $targets = array_unique($targets[1]);
        // add targets
        if(empty($targets)){
            $form->add_target($this->admin_email());
        } else {
            $form->set_targets($targets);
        }
        // delete them from tag
        $rest = preg_replace($targets_pattern, '', $tag); 
        
        // parse parameters
        preg_match_all($param_pattern, $rest, $params, PREG_SET_ORDER);
        if(empty($params)){
            $default = $this->settings('default_params');
            preg_match_all($param_pattern, ': ' . $default, $params, PREG_SET_ORDER);
        }
        // add fields
        foreach($params as $id => $param) {
            $field = new P01contact_field($form, $id, $param[1]);
            $field->set_title($param[4]);
            $field->set_value($param[7]);
            $field->set_required($param[2] == '!' ? True : False);
            $field->set_locked($param[6] == '=&gt;' ? True : False);
            $form->add_field($field);
        }
        
        return $form;
    }
    
    /**
     * Update POSTed form and try to send mail
     *
     * Check posted data, update form data,
     * define fields errors and form status.
     * At least, if there is no errors, try to send mail.
     */
    private function post()
    {
        // check posted data and update form data
        $form_id = $_POST['p01contact_form'];
        if(isset($this->forms[$form_id]))
        {
            $form = $this->forms[$form_id];
            foreach($this->format_data($_POST['p01contact_fields']) as $field_id => $field_post)
            {
                $field = $form->get_field($field_id);
                $field->set_value($field_post);
                $check = $field->check_content();
                $field->set_error($check);
                if($check) $errors = True;
            }
            // try to send mail
            if(@!$errors)
            {
                $tgets = $form->get_targets();
                if($this->settings('enable') === False) {
                    $form->set_status('disable');
                } elseif(empty($tgets)) {
                    $form->set_status('target');
                } else {
                    $form->send_mail();
                }
            }
         }
    }
    
    /**
     * Return next accessible form ID
     * @param string $key the setting key
     * @return mixed the setting value
     */
    private function new_form_id()
    {
        end($this->forms);
        $id = key($this->forms) + 1;
        reset($this->forms);
        return $id;
    }
    
    /**
     * Print POST and p01contact content.
     */
    public function debug()
    {
        echo'<h2 style="color:#c33">p01-contact plugin debug</h2>';
        if(!empty($_POST)) {
            echo'<h3>$_POST :</h3>';
            echo'<pre>';
            @print_r($_POST);
            echo'</pre>';
        }
        echo'<h3>$p01contact :</h3>';
        echo'<pre>';
        print_r($this);
        echo'</pre>';
    }
    
    /**
     * Return a setting value from config file
     * @param string $key the setting key
     * @return mixed the setting value
     */
    public function settings($key)
    {
        require 'p01-contact/config.php';
        if(isset($p01contact_settings[$key]))
            return $p01contact_settings[$key];
    }
    
    /**
     * Return GS admin email
     * @return string
     */
    public function admin_email()
    {
        $data = getXML(GSDATAOTHERPATH . 'user.xml');
        return $data->EMAIL;
    }
    
    /**
     * Format array values
     *
     * For aesthetic and security, and recursive.
     * @param array $array
     * @return array
     */
    function format_data($array)
    {
        foreach($array as $key => $val){
            if(is_array($val)) $this->format_data($array[$key]);
            else {
                $tmp = stripslashes($val);
                $tmp = htmlentities($tmp, ENT_QUOTES, 'UTF-8');
                $array[$key] = $tmp;
            }
        }
        return $array;
    }
    
    /**
     * Return a traduction of the keyword
     *
     * Manage languages between requested langs and existing traductions.
     * @param string $key the keyword
     * @return string
     */
    public function lang($key)
    {
        global $p01contact_lang, $p01contact_settings, $LANG;
        
        $lang_prefix = substr($LANG, 0, 2);
        
        $lang = empty($p01contact_settings['lang']) ? $lang_prefix : $p01contact_settings['lang'];
        $path = GSPLUGINPATH . 'p01-contact/lang/' . $lang . '.php';
        $lang = file_exists($path) ? $lang : 'en';
        
        include_once $path;
        
        if(isset($p01contact_lang[$key])) {
            return $p01contact_lang[$key];
        } else {
            return ucfirst($key);
        }
    }

    /**
     * Save settings if necessary and display configuration panel content
     *
     * Parse and replace values in php config file by POST.
     * Called by action handler function, called by GS in register_plugin().
     */
    function panel()
    {
        if(isset($_POST['p01-contact'])){
            $data = $this->format_data($_POST);
            $path = GSPLUGINPATH . 'p01-contact/config.php';
            if($content = @file_get_contents($path)) {
            
                $enable  = isset($data['enable']) ? 'True' : 'False';
                $content = preg_replace("`('enable' => )(True|False)`", "\\1$enable", $content);
                $debug   = isset($data['debug']) ? 'True' : 'False';
                $content = preg_replace("`('debug' => )(True|False)`", "\\1$debug", $content);
                $content = preg_replace("`('lang' => ')[a-z]*'`", "\\1{$data['lang']}'", $content);
                $content = preg_replace("`('default_params' => ')[^']*'`", "\\1{$data['default_params']}'", $content);
                $content = preg_replace("`'message_len' => [0-9]+`", "'message_len' => {$data['message_len']}", $content);
                
                foreach($data['blacklist'] as $key => $val){
                    $content = preg_replace("`('blacklist_$key' => ')[^']*'`", "\\1$val'", $content);
                }
                if(file_exists($path) && $file = @fopen($path, 'w')) {
                    fwrite($file, $content);
                    fclose($file);
                    
                    global $p01contact_settings;
                    require($path);
                    $success = 'Your changes were saved successfully.';
                } else {
                $i18n['ERROR'] = 'Can not modify config file';
                $error = 'check file permissions.<pre>' . $path . '</pre>';
                }
                
            } else {
                $i18n['ERROR'] = 'Can not open config file';
                $error = 'check if file exists and file permissions.<pre>' . $path . '</pre>';
            }
        }
        include(GSADMINPATH . 'template/error_checking.php');
        echo $this->panel_content();
    }

    /**
     * Return configuration panel content
     *
     * Display informations, parse config file and display settings form.
     * @return string
     */
    function panel_content()
    {
        global $p01contact_settings;
        $config_file = file_get_contents(GSPLUGINPATH . 'p01-contact/config.php');
        $pattern = '`/\*[^*]*\* ([^*]*).*\* ([^*]*)[^*]*(\* ([^*]*))?\*/`';
        preg_match_all($pattern, $config_file, $descs);

        /**
        * Return the parameter label
        * @return string
        */
        function label($d, $id){
            // d[1] => comment first line : title (red)
            // d[2] => comment second line : subtitle (italic)
            // d[4] => comment third line : warning (bold)
            $c = '<p><label style="float:none">' . $d[1][$id] . '</label>';
            $c.= '<i>' . $d[2][$id] . '</i><br />';
            if(@$d[4][$id]) $c.= '<b>' . $d[4][$id] . '</b><br />';
            $c.= '<br />';
            return $c;
        }
        
        $c = '<h2>p01-contact plugin configuration</h2>';
        $c.= '<p><a href="http://nliautaud.fr/wiki/travaux/getsimple_p01-contact">Documentation</a>';
        $c.= '- <a href="http://get-simple.info/extend/plugin/p01-contact/35">Last Version</a>';
        $c.= '- <a href="http://get-simple.info/forum/topic/1108">Help</a></p>';

        $c.= '<form action="' . $_SERVER ['REQUEST_URI'] . '" method="post">';
        
        $c.= label($descs, 0);
        $c.= '<input id="enable" type="checkbox" name="enable" ';
        $c.= $this->settings('enable') ? 'checked="checked" ' : '';
        $c.= '/></p>';
        
        $c.= label($descs, 1);
        $c.= '<input id="lang" type="text" name="lang" size=2 maxlength=2 ';
        $c.= 'value="' . $this->settings('lang') . '" /></p>';
        
        $c.= label($descs, 2);
        $c.= '<textarea id="default_params" name="default_params" style="height:40px">';
        $c.= $this->settings('default_params');
        $c.= '</textarea></p>';
        
        $c.= label($descs, 3);
        $c.= '<input id="message_len" type="text" name="message_len" size=3 maxlength=3 ';
        $c.= 'value="' . $this->settings('message_len').'" /></p>';
        
        $c.= label($descs, 4);
        foreach(array('name','email','address','phone','website','subject','message') as $field) {
            $content = $this->settings('blacklist_'.$field);
            $c.= ucfirst($field);
            $c.= '<textarea name="blacklist['.$field.']" ';
            $c.= 'style="height:'.(40+strlen($content)*0.2).'px">';
            $c.= $content . '</textarea>';
        }
        $c.= '</p>';
        
        $c.= label($descs, 5);
        $c.= '<input id="debug" type="checkbox" name="debug" ';
        $c.= $this->settings('debug') ? 'checked="checked" ' : '';
        $c.= '/></p>';
        
        $c.= '<input type="submit" name="p01-contact" value="Save settings" />';
        $c.= '</form>';
        
        return $c;
    }
}

/*
 * Contact form class
 
 * Contains fields, manage mail sending.
 */
class P01contact_form 
{    
    private $P01contact;
    
    private $id;
    private $status;
    private $targets;
    private $fields;

    /*
     * @param P01contact $P01contact
     * @param int $id the form id
     */
    public function __construct($P01contact, $id) 
    { 
        $this->P01contact = $P01contact;
        
        $this->id = $id;
        $this->status = '';
        $this->targets = array();
        $this->fields = array();
    }
    
    /*
     * Return the html display of the form
     * @return string the <form>
     */
    public function html() 
    {     
        $html  = '<form action="#p01-contact' . $this->id . '" ';
        $html .= 'id="p01-contact' . $this->id . '" class="p01-contact" method="post">';
    
        $html .= $this->html_status();
        
        if($this->status != 'sent') {
            foreach($this->fields as $id => $field) $html .= $field->html();
        
            $html .= '<div><input name="p01contact_form" type="hidden" value="' . $this->id . '" />';
            $html .= '<input class="submit" ';
            $html .= 'type="submit" value="' . $this->lang('send') . '" /></div>';
        }
        $html .= '</form>';
    
        return $html;
    }
    
    /*
     * Return an html display of the form status
     * @return string the <div>
     */
    private function html_status() 
    { 
        if(!$this->status) return '';
        $style = '
	        margin:0 0 20px 0;
	        background:#FCFBB8; 
	        line-height:30px;
	        padding:0 10px;
	        border:1px solid #F9CF51;
	        border-radius: 5px;
	        -moz-border-radius: 5px;
	        -khtml-border-radius: 5px;
	        -webkit-border-radius: 5px;';
        $style .= $this->status == 'sent' ? 'color:#308000;' : 'color:#D94136;';
        
        return '<div style="'.$style.'">' . $this->lang($this->status) . '</div>';
    }
    
    /*
     * Return an html http:// link
     * @param string $href the link address
     * @param string $title if not used, the link title will be the address
     * @return string the <a>
     */   
    private function html_link($href, $title = False)
    {
        if(!$title) $title = $href;
        return '<a href="http://' . $href . '">' . $title . '</a>';
    }

    /*
     * Return an html mailto: link
     * @param string $href the email
     * @param string $title if not used, the link title will be the email
     * @return string the <a>
     */  
    private function html_mail_link($href, $title = False)
    {
        if(!$title) $title = $href;
        return '<a href="mailto:' . $href . '">' . $title . '</a>';
    }

    /**
     * Send a mail based on form
     *
     * Create the mail content and headers along to plugin settings, form
     * and fields datas; and update the form status (sent|error).
     */
    function send_mail()
    {   
        $server = $_SERVER['SERVER_NAME'];
        $uri = $_SERVER['REQUEST_URI'];
        
        // title
        $content  = '<h2>' . $this->lang('fromsite') . ' <i>' . $_SERVER['SERVER_NAME'] . '</i></h2>';
        $content .= '<h3>' . date('r') . '</h3><br/>';
        
        // fields
        $skip = array('captcha');
        foreach($this->fields as $field)
        {
            $type  = $field->get_type();
            $value = $field->get_value();
            $title = $field->get_title();
            if($type == 'name')
                $name = $value;
            elseif($type == 'email')
                $email = $value;
            elseif($type == 'subject')
                $subject = $value;
            elseif(
                !in_array($type, $skip)
                && !empty($value))
            {
                $title = !empty($title) ? $title : $type;
                $content .= '<p><b>' . $this->lang($title).' :</b> ';
                switch($type){
                    case 'message' :
                    case 'textarea' :
                        $content .= '<p style="margin:10px;padding:10px;border:1px solid silver">';
                        $content .= nl2br($value) . '</p>';
                        break;
                    case 'website' :
                        $content .= $this->html_link($value);
                        break;
                    default :
                        $content .=  $value;
                } 
                $content .= '</p>';
            }
        }
        
        // footer infos
        $content .= '<p><i>' . $this->lang('sentfrom');
        $content .= ' ' . $this->html_link($server.$uri, $uri);
        
        // prevention message
        if(@!$askcopy) {
            $content .= '<br />If this mail should not be for you, please contact ';
            $content .= $this->html_mail_link($this->P01contact->admin_email());
        }
        $content .= '</i></p>';
        
        $targets = implode(',', $this->targets);
        
        if(empty($name)) $name = $this->lang('nofrom');
        if(empty($email)) $email = $this->lang('nofrom');
        if(empty($subject)) $subject = $this->lang('nosubject');
        $subject = '=?utf-8?B?' . base64_encode($subject) . '?=';
        
        $headers  = "From: $name <$email>\r\n";
        $headers .= "Reply-To: $name <$email>\r\n";
        $headers .= "Return-Path: $name <$email>\r\n";
        $headers .= "MIME-Version: 1.0\r\n";
        $headers .= "Content-type: text/html; charset=UTF-8\r\n";
        $headers .= "X-Mailer: PHP/" . phpversion() . "\r\n" ;
        
        if(!$this->settings('debug')) {
            // send mail
            $sent = mail($targets, $subject, $content, $headers);
            if(isset($askcopy) && $askcopy) {
                mail($email, $subject, $content, $headers);
            }
            if(@$sent) {
                $this->status = 'sent';
            } else {
                $this->status = 'error';
            }
        } else {
            // display mail for debug
            echo'<h2 style="color:#c33">p01-contact (not) sent mail :</h2>';
            echo'<pre>' . $headers . '</pre>';
            echo'<div style="border:1px solid black">' . $content . '</div>';
            $this->status = 'Debug mode. See top of the page.';
        }            
    }

    /**
     * GETTERS / SETTERS
     */
    
    public function get_targets() {
        return $this->targets;
    }
    public function get_target($id) {
        return $this->targets[$id];
    }
    public function add_target($target) {
        array_push($this->targets, $target);
    }
    public function set_targets(array $targets) {
        $this->targets = $targets;
    }
    
    public function get_field($id) {
        return $this->fields[$id];
    }
    public function get_fields() {
        return $this->fields;
    }
    public function add_field(P01contact_field $field) {
        array_push($this->fields, $field);
    }
    
    public function set_status($status) {
        if(is_string($status)) $this->status = $status;
    }
    
    public function get_id() {
        return $this->id;
    }
    
    public function settings($key) {
        return $this->P01contact->settings($key);
    }
    public function lang($key) {
        return $this->P01contact->lang($key);
    }
} 

class P01contact_field 
{
    private $form;
    
    private $id;
    private $type;
    private $title;
    private $value;
    private $required;
    private $locked;
    private $error;

    /*
     * @param P01contact_form $form the container form
     * @param int $id the field id
     * @param string $type the field type
     */
    public function __construct($form, $id, $type) 
    { 
        $this->form = $form;
        
        $this->id = $id;
        $this->type = $type;
        $this->title = '';
        $this->value = '';
        $this->required = False;
        $this->locked = False;
        $this->error = '';
    }
    
    /**
     * Check field value
     *
     * Check if field is empty and required or
     * not empty but not valid.
     * @return string the error key, or empty
     */
    public function check_content()
    {
        if(empty($this->value) && $this->required) {
            // empty and required
            return 'field_required';
        }
        elseif(!empty($this->value) && !$this->check_validity()) {
            // not empty but not valid
            return 'field_' . $this->type;
        }
        else return '';
    }

    /**
     * Check if field value is valid
     * @return boolean
     */
    public function check_validity()
    {
        if($this->blacklisted()) return False;
        
        switch($this->type) {
            case 'email':
                $pattern = '`^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$`i';
                if(preg_match($pattern, $this->value)) return True;
                else return False;
            case 'phone':
                $pattern = '`^\+?[-0-9(). ]{6,}$$`i';
                if(preg_match($pattern, $this->value)) return True;
                else return False;
            case 'website':
                $pattern = "`^((http|https|ftp):\/\/(www\.)?|www\.)[a-zA-Z0-9\_\-]+\.([a-zA-Z]{2,4}|[a-zA-Z]{2}\.[a-zA-Z]{2})(\/[a-zA-Z0-9\-\._\?\&=,'\+%\$#~]*)*$`i";
                if(preg_match($pattern, $this->value)) return 1;
                else return False;
            case 'message':
                $size = strlen($this->value);
                if($size > $this->form->settings('message_len')) return True;
                else return False;
            case 'captcha':
                include_once 'plugins/p01-contact/captcha/securimage.php';
                $securimage = new Securimage();
                if ($securimage->check($this->value) == False) return False;
                else return True;   
            default:
                return True;
        }
    }

    /**
     * Check if field value is blacklisted
     *
     * Search any entry of config file field type
     * blacklist in field value.
     * @return boolean
     */
    public function blacklisted()
    {
        $list = $this->form->settings('blacklist_' . $this->type);
        
        if(empty($list)) {
            return False;
        }
        $array = explode(',', $list);
        foreach($array as $avoid){
            if(preg_match('`' . $avoid . '`', $this->value))
                return True;
        }
        return False;
        
    }
    
    /*
     * Return the html display of the field
     *
     * Manage field title, error message, and type-based display
     * @return string the <div>
     */
    public function html() 
    {   
        $id  = 'p01-contact' . $this->form->get_id() . '_field' . $this->id;

        $html  = '<div>';
        $html .= '<label for="' . $id . '">';
        $html .= !empty($this->title) ? $this->title : $this->form->lang($this->type);
        $html .= $this->required ? ' <strong>*</strong>' : '';
        // field error message
        if(!empty($this->error)) {
            $html .= ' <span style="font-size:0.7em;color:red">';
            $html .= $this->form->lang($this->error);
            $html .= '</span>';
        }
        $html .= '</label>';
        
        $html .= '<br/>';
        
        if($this->special_type()){
            $type = $this->special_type();
        } else {
            $type = $this->type;
        }
        
        $name = 'p01contact_fields[' . $this->id . ']';
        switch($type) {
            case 'textarea' :
                $html .= '<textarea id="' . $id . '" ';
                $html .=     'name="' . $name . '" rows="10"';
                $html .=     $this->locked ? ' disabled="disabled"' : '';
                $html .= '>' . $this->value . '</textarea>';
                break;
            case 'captcha' :
                $html .= '<img id="captchaimg" src="plugins/p01-contact/captcha/securimage_show.php" ';
                $html .=     'alt="CAPTCHA Image" /><br/>';
                $html .= '<a href="#"';
                $html .=     'onclick="document.getElementById(\'captchaimg\').src = \'plugins/p01-contact/captcha/securimage_show.php?\' ';
                $html .=     '+ Math.random(); return false">';
                $html .=     $this->form->lang('reload');
                $html .= '</a> ';
                $html .= '<input id="' . $id . '" ';
                $html .=     'type="text" name="' . $name . '" ';
                $html .=     'size="10" maxlength="6"';
                $html .=     $this->locked ? ' disabled="disabled" ' : ' ';
                $html .= ' />';
                break;
            case 'file' :
                $html .= '<input id="' . $id . '" ';
                $html .=     'name="' . $name . '" type="file"';
                $html .=     $this->locked ? ' disabled="disabled" ' : ' ';
                $html .= ' />';
                break;
            case 'checkbox' :
                $html .= '<input id="' . $id . '" ';
                $html .=     'name="' . $name . '" type="checkbox"';
                $html .=     $this->locked ? ' disabled="disabled" ' : ' ';
                $html .=     $this->value == 'on' ? 'checked' : '';
                $html .= ' />';
                break;
            case 'text' :
                $html .= '<input id="' . $id . '" ';
                $html .=     'name="' . $name . '" type="text" ';
                $html .=     'value="' . $this->value . '"';
                $html .=     $this->locked ? ' disabled="disabled" ' : ' ';
                $html .= ' />';
                break;
        }
        $html .= '</div>';
        return $html;
    }
        
    /**
     * If the field is special return its type, or return False.
     */
    function special_type()
    {
        $types = array(
            'name'    => 'text',
            'email'   => 'text',
            'phone'   => 'text',
            'website' => 'text',
            'subject' => 'text',
            'address' => 'textarea',
            'message' => 'textarea',
            'file'    => 'file',
            'captcha' => 'captcha',
            'askcopy' => 'checkbox');
        if(isset($types[$this->type])) {
            return $types[$this->type];
        } else {
            return False;
        }
    }
    
    /**
     * GETTERS / SETTERS
     */
     
    public function get_type(){
        return $this->type;
    }
    
    public function set_title($title){
        if(is_string($title)) $this->title = $title;
    }
    public function get_title(){
        return $this->title;
    }
    
    public function set_value($value){
        if(is_string($value)) $this->value = $value;
    }
    public function get_value(){
        return $this->value;
    }
    
    public function set_required($required){
        if(is_bool($required)) $this->required = $required;
    }
    public function set_locked($locked){
        if(is_bool($locked)) $this->locked = $locked;
    }
    public function set_error($error){
        if(is_string($error)) $this->error = $error;
    }
}

